'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();
// import i18n from '../models/i18n'

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _express = require('express');

var _express2 = _interopRequireDefault(_express);

var _handlebars = require('handlebars');

var _handlebars2 = _interopRequireDefault(_handlebars);

var _handlebarsIntl = require('handlebars-intl');

var _handlebarsIntl2 = _interopRequireDefault(_handlebarsIntl);

var _htmlMinifier = require('html-minifier');

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _ = require('../');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Page class
 * manage HTML generation for page template
 */

var Page = function () {

  /**
   * Create new page object
   * @param  {Object} params req.params from express route
   * @param  {Object} i18n tranduction
   * @param  {Function} callback 
   * @param  {Boolean} onlyHTML default = false, if true HTML content will contains abe attributs
   * @return {String} HTML page as string
   */

  function Page(path, text, json) {
    var _this = this;

    var onlyHTML = arguments.length <= 3 || arguments[3] === undefined ? false : arguments[3];

    _classCallCheck(this, Page);

    _handlebars2.default.registerHelper('abe', _.compileAbe); // HandlebarsJS unique text helper
    _handlebars2.default.registerHelper('moduloIf', _.moduloIf); // HandlebarsJS helper for modulo test
    _handlebars2.default.registerHelper('testObj', _.testObj); //
    _handlebars2.default.registerHelper('closeBlock', _.closeBlock); //
    _handlebars2.default.registerHelper('subBlock', _.subBlock); //
    _handlebars2.default.registerHelper('openBlock', _.openBlock); //
    _handlebars2.default.registerHelper('attrAbe', _.attrAbe); //
    _handlebars2.default.registerHelper('isSubBlock', _.isSubBlock); //
    _handlebars2.default.registerHelper('notEmpty', _.notEmpty);
    _handlebars2.default.registerHelper('printJson', _.printJson);
    _handlebars2.default.registerHelper('className', _.className);
    _handlebars2.default.registerHelper('folders', _.folders);
    _handlebars2.default.registerHelper('printConfig', _.printConfig);
    _handlebarsIntl2.default.registerWith(_handlebars2.default);

    var util = new _.Util();
    var intlData = _.config.intlData;

    var pattTag = /(>|\t|\n|\r| )({{abe.*type=[\'|\"][text|rich].*}})/g;

    if (!onlyHTML) {
      text = text.replace(pattTag, '$1<abe>$2</abe>');
    }

    _.abeEngine.instance.content = json;
    _.log.write('Index-new-page', 'json', json);
    text = text.replace(/\{\{#each (\w+)(.)*?\}\}/g, '{{#each $1}}');
    var pattNode = /(>|\t|\n|\r| )({{abe.*type=[\'|\"][text|rich].*}})/g,
        pattAttr = /( [A-Za-z0-9\-\_]+="*{{.*?}})/g,
        pattSpeAttr = /=[\'|\"]{{abe.*}}[\'|\"]/g,
        pattEach = />\s*(\{\{#each (\r|\t|\n|.)*?\/each\}\})/g,
        arrayEach = [],
        match,
        textEach,
        textSpeAttr;

    var insertAbeEach = function insertAbeEach(theMatch, keyArray, lastIndex) {
      var matchBlock = theMatch[0];
      if (util.isEachStatement(matchBlock)) return;
      if (util.isBlockAbe(matchBlock)) {
        var more_attr = '';
        if ((0, _.getAttr)(matchBlock, 'key_text') !== '') {
          var more_key_attr = (0, _.getAttr)(match[0], 'key_text').split('.');
          more_key_attr = more_key_attr[more_key_attr.length - 1];
          more_attr = ' data-abe-text="' + keyArray + '[index]-' + more_key_attr + '" ';
        }
        if ((0, _.getAttr)(matchBlock, 'key_text') !== '') {
          var matchIndex = text.indexOf(matchBlock) + matchBlock.length;
          var index = matchIndex + /(\n|.)*?>/g.exec(text.slice(matchIndex))[0].length;
          var start = text.slice(0, index);
          var end = text.slice(index);
          var abeText = '{{abe  dictionnary=\'' + keyArray + '\'\n                                type=\'text\'\n                                key=\'' + (0, _.getAttr)(matchBlock, 'key_text') + '\'\n                                desc=\'' + (0, _.getAttr)(matchBlock, 'desc') + '\'\n                                max-length=\'' + (0, _.getAttr)(matchBlock, 'max-length') + '\'}}';
          text = start + abeText + end;
        }

        var newMatchBlock = (!onlyHTML ? (/=[\"\']\{\{(.*?)\}\}/g.test(matchBlock) ? ' data-abe-attr="' + matchBlock.split('=')[0].trim() + '"' : '') + ' data-abe="' + (0, _.getAttr)(matchBlock, 'key').replace('.', '[index].') + '" ' + more_attr + matchBlock : matchBlock).replace(new RegExp('(key=[\'|"])' + keyArray + '.', 'g'), '$1' + keyArray + '[index].').replace(/\{\{abe/, '{{abe dictionnary=\'' + keyArray + '\'');

        text = text.replace(matchBlock, newMatchBlock);
      }
    };

    while (textEach = pattEach.exec(text)) {
      var keyArray = textEach[0].match(/#each (\n|.)*?\}/);
      keyArray = keyArray[0].slice(6, keyArray[0].length - 1);
      // insert span caché avec le contenu echapé du element bloc pour l'ajout dynamique de bloc dans la phase d'édition
      var dataBlock = textEach[0].replace(/(>\s*\{\{#each (\n|.)*?\}\})/, '').replace('{{/each}}', '');
      if (!onlyHTML) {
        var meta = _.config.meta.name;
        var template = _handlebars2.default.compile(dataBlock.replace(/{{abe(.*?)}}/g, '[[abe$1]]').replace(new RegExp('\\.\\./' + meta, 'g'), meta), { noEscape: true });

        var insertCompiled = template(json, { data: { intl: intlData } }).replace(/\[\[abe(.*?)\]\]/g, '{{abe$1}}');
        text = text.replace(textEach[0], textEach[0] + ('<span class="hidden-abe insert-' + keyArray + '">' + util.encodeAbe(insertCompiled) + '</span>'));
      }
      while (match = pattNode.exec(textEach[0])) {
        insertAbeEach(match, keyArray, pattEach.lastIndex - textEach[0].length);
      }
      while (match = pattAttr.exec(textEach[0])) {
        insertAbeEach(match, keyArray, pattEach.lastIndex - textEach[0].length);
      }
    }
    var matches = [];
    while (match = pattNode.exec(text)) {
      if (matches.indexOf(match[0]) === -1) {
        matches.push(match[0]);
      }
    }
    Array.prototype.forEach.call(matches, function (match) {
      text = _this._abeFunc(text, match, onlyHTML);
      if (!onlyHTML) {
        text = text.replace((0, _.escapeTextToRegex)(match, 'g'), ' data-abe="' + (0, _.getAttr)(match, 'key').replace(/\./g, '-') + '" ' + match);
      }
    });
    while (match = pattAttr.exec(text)) {
      // While regexp match {{attribut}}, ex: link, image ...
      if (util.isSingleAbe(match[0], text)) {
        var more_attr = '';
        if ((0, _.getAttr)(match[0], 'key_text') !== '') {
          more_attr = ' data-abe-text="' + (0, _.getAttr)(match[0], 'key_text') + '" ';
        }
        if (!onlyHTML) {
          text = text.replace(new RegExp(match[0]), ' data-abe-attr="' + match[0].split('=')[0].trim() + '"' + ' data-abe="' + (0, _.getAttr)(match, 'key').replace(/\./g, '-') + '"' + more_attr + match[0].replace('}}', ' has-abe=1}}'));
        }
        if ((0, _.getAttr)(match[0], 'key_text') !== '') {
          var match_more_attr = text.slice(match.index).match(/.*<\//)[0];
          text = text.replace(match_more_attr, match_more_attr.slice(0, match_more_attr.length - 2) + ('{{abe  type=\'text\'\n                                      key=\'' + (0, _.getAttr)(match[0], 'key_text') + '\'\n                                      desc=\'' + (0, _.getAttr)(match[0], 'desc') + '\'\n                                      max-length=\'' + (0, _.getAttr)(match[0], 'max-length') + '\'}}</'));
        }
      }
    }

    var source = _.config.source.name;
    if (!onlyHTML && typeof json[source] !== 'undefined' && json[source] !== null) {
      var keys = Object.keys(json[source]);
      for (var i in keys) {
        var patAttrSource = new RegExp(' ([A-Za-z0-9\-\_]+)=["|\'].*?({{' + keys[i] + '}}).*?["|\']', 'g');
        var patAttrSourceMatch = text.match(patAttrSource);

        if (typeof patAttrSourceMatch !== 'undefined' && patAttrSourceMatch !== null) {
          var patAttrSourceInside = new RegExp('(\\S+)=["\']?((?:.(?!["\']?\\s+(?:\\S+)=|[>"\']))+.)["\']?({{' + keys[i] + '}}).*?["|\']', 'g');
          Array.prototype.forEach.call(patAttrSourceMatch, function (pat) {
            var patAttrSourceCheck = patAttrSourceInside.exec(pat);
            if (typeof patAttrSourceCheck !== 'undefined' && patAttrSourceCheck !== null) {
              var checkEscaped = /["|'](.*?)["|']/;
              checkEscaped = checkEscaped.exec(patAttrSourceCheck[0]);
              if (typeof checkEscaped !== 'undefined' && checkEscaped !== null && checkEscaped.length > 0) {
                checkEscaped = escape(checkEscaped[1]);
                text = text.replace(patAttrSourceCheck[0], ' data-abe-attr="' + patAttrSourceCheck[1] + '" data-abe-attr-escaped="' + checkEscaped + '" data-abe="' + keys[i] + '" ' + patAttrSourceCheck[0]);
              }
            }
          });
        }

        text = text.replace(new RegExp('{{(' + keys[i] + ')}}(?=(?:[^"]*"[^"]*")*[^"]*$)', 'g'), '<abe data-abe="$1" >{{$1}}</abe>');
        var eachSource = new RegExp('{{#each ' + keys[i] + '}[\\s\\S a-z]*?{{/each}}');
        var matches = eachSource.exec(text);
        if (typeof matches !== 'undefined' && matches !== null) {
          if (!onlyHTML) {
            text = text.replace(matches[0], '<abe>' + matches[0] + '<span class="hidden-abe select-' + keys[i] + '">' + util.encodeAbe(matches[0]) + '</span></abe>');
          } else {
            text = text.replace(matches[0], matches[0] + '<span class="hidden-abe select-' + keys[i] + '">' + util.encodeAbe(matches[0]) + '</span>');
          }
        }
      }
    }

    text = text.replace(/\[index\]\./g, '[{{@index}}]-');
    if (typeof text !== "undefined" && text !== null && typeof text.replace === "function") {
      var template = _handlebars2.default.compile(!onlyHTML ? util.insertDebugtoolUtilities(text) : text, { noEscape: true });

      var tmp = template(json, {
        data: { intl: intlData }
      });
      _.log.delAndWrite('Index-new-page', 'result', text, json, tmp);
      this.html = tmp;
    }
  }

  /**
   * add handlebar {{if variable empty}} state if onlyHTML or "data-if-empty-clear" attribute
   * 
   * @param  {String} text      html
   * @param  {String} key       abe
   * @param  {String} tag       name
   * @param  {String} match     regex string to match
   * @param  {Boolean} onlyHTML boolean for generated html or browser
   * @return {String}           new text with if or data attributes
   */

  _createClass(Page, [{
    key: '_abeClear',
    value: function _abeClear(text, key, tag, match, onlyHTML) {
      var hideTagRegex;
      var hideHtmls = (0, _.getEnclosingTags)(text, match, tag);

      Array.prototype.forEach.call(hideHtmls, function (hideHtml) {
        if (onlyHTML) {
          hideTagRegex = (0, _.escapeTextToRegex)(hideHtml, 'gm');
          text = text.replace(hideTagRegex, '{{#if ' + key + '}}' + hideHtml + '{{/if}}');
        } else {
          var firstTag = /(<[^\s>]+)/.exec(hideHtml);
          firstTag = firstTag[0];

          var hideHtmlWithAttr = hideHtml.replace(firstTag, firstTag + ' data-if-empty-clear="' + key + '"');
          text = text.replace((0, _.escapeTextToRegex)(hideHtml, 'g'), hideHtmlWithAttr);
        }
      });

      return text;
    }

    /**
     * check if abe tag own a custom attribute
     * 
     * @param  {String} text      html
     * @param  {String} tag       name
     * @param  {String} match     regex string to match
     * @return {String}           new text with if or data attributes
     *
     * for example :
     *
     * {{abe type="" if-empty="clear(something)"}}
     *
     * will call methode abeClear on this tag
     */

  }, {
    key: '_abeFunc',
    value: function _abeFunc(text, match, onlyHTML) {
      var ifEmpty = (0, _.getAttr)(match, 'if-empty');
      var key = (0, _.getAttr)(match, 'key');

      if (ifEmpty !== '') {
        var tag = /\(([^)]+)\)/g.exec(ifEmpty);
        var func = /([^)]+)\(/g.exec(ifEmpty);

        if (typeof func[1] !== 'undefined' && func[1] !== null) {
          switch (func[1]) {
            case 'clear':
              if (typeof tag[1] !== 'undefined' && tag[1] !== null) {
                text = this._abeClear(text, key, tag[1], match, onlyHTML);
              }
              break;
          }
        }
      }

      return text;
    }
  }]);

  return Page;
}();

exports.default = Page;