'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.save = save;
exports.saveJson = saveJson;
exports.saveHtml = saveHtml;
exports.dateIso = dateIso;

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _extend = require('extend');

var _extend2 = _interopRequireDefault(_extend);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _ = require('../');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function save(url, tplPath) {
  var json = arguments.length <= 2 || arguments[2] === undefined ? null : arguments[2];
  var text = arguments.length <= 3 || arguments[3] === undefined ? '' : arguments[3];
  var type = arguments.length <= 4 || arguments[4] === undefined ? '' : arguments[4];

  var tplUrl = _.FileParser.getFileDataFromUrl(url);
  var type = type || _.FileParser.getType(url);
  var path = dateIso(tplUrl, type);
  var tpl = tplPath.replace(_.config.root + '/', '');
  var fullTpl = _.fileUtils.concatPath(_.config.root, _.config.templates.url, tpl) + '.' + _.config.files.templates.extension;

  if (typeof json === 'undefined' || json === null) {
    if (_.fileUtils.isFile(tplUrl.json.path)) {
      json = _fsExtra2.default.readJsonSync(tplUrl.json.path);
    } else {
      json = {};
    }
  }

  var ext = {
    template: tpl,
    link: tplUrl.publish.link,
    type: type
  };

  var meta = _.config.meta.name;
  json[meta] = (0, _extend2.default)(json[meta], ext);
  _.Util.addMetas(tpl, json, type);

  if (typeof text === 'undefined' || text === null || text === '') {
    text = _fsExtra2.default.readFileSync(fullTpl, 'utf8');
  }
  _.Util.getDataList(_.FileParser.getWebsite(url), text, json);
  text = _.Util.removeDataList(text);

  var page = new _.Page(tplUrl.publish.path, text, json, true);
  saveJson(path.jsonPath, json);
  saveHtml(path.htmlPath, page.html);

  _.FileParser.copySiteAssets();

  if (type === 'validate') {
    _fsExtra2.default.removeSync(tplUrl.json.path);
    _fsExtra2.default.removeSync(tplUrl['submit'].path);
  }

  return {
    json: json,
    html: page.html,
    jsonPath: path.jsonPath,
    htmlPath: path.htmlPath
  };
}

function saveJson(url, json) {
  _mkdirp2.default.sync(_.fileUtils.removeLast(url));

  _fsExtra2.default.writeJsonSync(url, json, {
    space: 2,
    encoding: 'utf-8'
  });
  _.log.delAndWrite('Index-saveJson', 'Success result writing file : ' + url, json);
}

function saveHtml(url, html) {
  _mkdirp2.default.sync(_.fileUtils.removeLast(url));
  _fsExtra2.default.writeFileSync(url, html);
  _.log.delAndWrite('Index-saveHtml', 'Success result writing file : ' + url, html);
}

function dateIso(tplUrl) {
  var type = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  var newDateISO;
  var dateISO;
  var validate;

  var saveJsonFile = tplUrl.json.path;
  var saveFile = tplUrl[type].path;
  var oldDateISO = _.fileAttr.get(saveFile).d;

  switch (type) {
    case 'validate':
      dateISO = 'v' + oldDateISO;
      break;
    case 'submit':
      dateISO = 'w' + oldDateISO;
      break;
    case 'draft':
      newDateISO = new Date().toISOString();
      dateISO = 'd' + newDateISO;
      break;
    case 'publish':
      saveJsonFile = tplUrl.publish.json;
      saveFile = tplUrl.publish.path;
      break;
    default:
      break;
  }

  if (dateISO) {
    saveJsonFile = _.fileAttr.add(saveJsonFile, dateISO);
    saveFile = _.fileAttr.add(saveFile, dateISO);
  }

  return {
    jsonPath: saveJsonFile,
    htmlPath: saveFile
  };
}