import fse from 'fs-extra'
import extend from 'extend'
import mkdirp from 'mkdirp'

import {
  Util,
  FileParser,
  config,
  fileUtils,
  fileAttr,
  cli,
  log,
  Page
} from '../'

export function save(url, tplPath, json = null, text = '', type = '') {
  var tplUrl = FileParser.getFileDataFromUrl(url)
  var type = type || FileParser.getType(url)
  var path = dateIso(tplUrl, type)
  var tpl = tplPath.replace(config.root + '/', '')
  var fullTpl = fileUtils.concatPath(config.root, config.templates.url, tpl) + '.' + config.files.templates.extension

  if(typeof json === 'undefined' || json === null) {
    if(fileUtils.isFile(tplUrl.json.path)) {
      json = fse.readJsonSync(tplUrl.json.path)
    }else {
      json = {}
    }
  }

  var ext = {
    template: tpl,
    link: tplUrl.publish.link,
    type: type
  }

  let meta = config.meta.name
  json[meta] = extend(json[meta], ext)
  Util.addMetas(tpl, json, type)

  if(typeof text === 'undefined' || text === null || text === '') {
    text = fse.readFileSync(fullTpl, 'utf8')
  }
  Util.getDataList(FileParser.getWebsite(url), text, json)
  text = Util.removeDataList(text)

  var page = new Page(tplUrl.publish.path, text, json, true)
  saveJson(path.jsonPath, json)
  saveHtml(path.htmlPath, page.html)

  FileParser.copySiteAssets()

  if(type === 'validate') {
    fse.removeSync(tplUrl.json.path)
    fse.removeSync(tplUrl['submit'].path)
  }

  return {
    json: json,
    html: page.html,
    jsonPath: path.jsonPath,
    htmlPath: path.htmlPath
  }
}

export function saveJson(url, json) {
  mkdirp.sync(fileUtils.removeLast(url))

  fse.writeJsonSync(url, json, {
    space: 2,
    encoding: 'utf-8'
  })
  log.delAndWrite('Index-saveJson', `Success result writing file : ${url}`, json)
}

export function saveHtml(url, html) {
  mkdirp.sync(fileUtils.removeLast(url))
  fse.writeFileSync(url, html)
  log.delAndWrite('Index-saveHtml', `Success result writing file : ${url}`, html)
}

export function dateIso(tplUrl, type = null) {
  var newDateISO
  var dateISO
  var validate

  var saveJsonFile = tplUrl.json.path
  var saveFile = tplUrl[type].path
  var oldDateISO = fileAttr.get(saveFile).d
  
  switch(type) {
    case 'validate':
      dateISO = 'v' + oldDateISO
      break;
    case 'submit':
      dateISO = 'w' + oldDateISO
      break;
    case 'draft':
      newDateISO = (new Date().toISOString())
      dateISO = 'd' + newDateISO
      break;
    case 'publish':
      saveJsonFile = tplUrl.publish.json
      saveFile = tplUrl.publish.path
      break;
    default:
      break;
  }

  if(dateISO) {
    saveJsonFile = fileAttr.add(saveJsonFile, dateISO)
    saveFile = fileAttr.add(saveFile, dateISO)
  }

  return {
    jsonPath: saveJsonFile,
    htmlPath: saveFile
  }
}