import fse from 'fs-extra'
import dircompare from 'dir-compare'
import mkdirp from 'mkdirp'
import moment from 'moment'

import {
	cli,
	log,
	fileAttr,
	config
} from '../'

export default class FileUtils {

	constructor() {}

	static pathWithRoot(path) {
		if(typeof path === 'undefined' || path === null || path === '') path = ''
		return FileUtils.concatPath(config.root, path.replace(config.root, '')).replace(/\/$/, "")
	}

	static cleanPath(path) {
		if(typeof path !== 'undefined' && path !== null) path = path.replace(/\/$/, "")
		return path
	}

	static concatPath() {
		var path = ''
		Array.prototype.forEach.call([].slice.call(arguments), (argument) => {
			if(path !== '') argument = argument.replace(/^\//, "")
		  path += FileUtils.cleanPath(argument) + '/'
		})

		path = FileUtils.cleanPath(path)

		return path
	}

	static removeLast(path) {
		return path.substring(0, FileUtils.cleanPath(path).lastIndexOf('/'))
	}

	static replaceExtension(path, ext) {
		return path.substring(0, path.lastIndexOf('.')) + '.' + ext
	}

	static removeExtension(path) {
		return path.substring(0, path.lastIndexOf('.'))
	}

	static filename(path) {
		return FileUtils.cleanPath(path).substring(FileUtils.cleanPath(path).lastIndexOf('/') + 1)
	}

	static isFile(path) {
		try{
			var stat = fse.statSync(path)
      return true
		}catch(e){
			return false
		}
		return false
	}

	static isValidFile(fileName) {
		var dotPosition = fileName.indexOf('.')
		if(dotPosition > 0) return true
		return false
	}

	static cleanTplName(path) {
		var cleanTplName = fileAttr.delete(path)
		cleanTplName = cleanTplName.replace(config.root, '')
		cleanTplName = cleanTplName.split('/')
		cleanTplName.shift()
		return cleanTplName.join('/')
	}

	static cleanFilePath(path) {
		var cleanFilePath = fileAttr.delete(path)
		cleanFilePath = cleanFilePath.replace(config.root + '/', '')
		cleanFilePath = cleanFilePath.split('/')
		cleanFilePath.shift()
		return cleanFilePath.join('/')
	}

  static getFilePath(path) {
  	var res = null
  	if(typeof path !== 'undefined' && path !== null && path !== '') {
  		res = path.replace(config.root + '/')
  		res = FileUtils.concatPath(config.root, config.draft.url, res)
  	}
  	return res
  }

  static getTemplatePath(path) {
  	var res = null
  	if(typeof path !== 'undefined' && path !== null && path !== '') {
  		res = path.replace(config.root + '/')
  		res = FileUtils.concatPath(config.root, config.templates.url, res)
  	}
  	return res
  }
  
  static mergeFiles(files1, files2) {
  	var merged = {}

  	Array.prototype.forEach.call(files1, (file1) => {
  		var cleanFilePath1 = file1.cleanFilePath
  		merged[cleanFilePath1] = [file1]
  		var date1 = new Date(file1.date)

  		Array.prototype.forEach.call(files2, (file2) => {
  			var cleanFilePath2 = file2.cleanFilePath
  			var date2 = new Date(file2.date)
				if(cleanFilePath1 === cleanFilePath2) {
	  			if(date2 >= date1) {
	  				merged[cleanFilePath1] = [file2]
	  			}else {
  					merged[cleanFilePath1].push(file2)
	  			}
  			}
  		})
  	})

  	Array.prototype.forEach.call(files2, (file2) => {
  		var cleanFilePath2 = file2.cleanFilePath

  		if(typeof merged[cleanFilePath2] === 'undefined' || merged[cleanFilePath2] === null) {
  			merged[cleanFilePath2] = [file2]
  		}
  	})

  	return merged
  }

}
