import express from 'express'
import fs from 'fs'
import fse from 'fs-extra'
import mkdirp from 'mkdirp'
import {minify} from 'html-minifier'
import extend from 'extend'

import {
  fileAttr,
  save,
  getAttr, getEnclosingTags, escapeTextToRegex,
  Util,
  FileParser,
  fileUtils,
  config,
  cli,
  log,
  Page,
  serveSite
} from '../../cli'

import {editor} from './editor'
import locale from '../helpers/abe-locale'

var router = express.Router()

// Abe templating engine route
router.get('/abe/*', function(req, res) {
  var templatePath = fileUtils.getTemplatePath(req.params[0])
  var filePath = fileUtils.getFilePath(req.query.filePath)
  var obj
  var tplUrl
  var isHome = true
  var manager = {}

  if(templatePath !== null && filePath !== null) {

    isHome = false

    if(!fileAttr.test(filePath)){
      var folderFilePath = filePath.split('/')
      folderFilePath.pop()
      folderFilePath = fileUtils.pathWithRoot(folderFilePath.join('/'))
      mkdirp.sync(folderFilePath)
      var files = FileParser.getFiles(folderFilePath, true, 2)
      var latest = fileAttr.filterLatestVersion(fileAttr.getFilesRevision(files, filePath), 'draft')
      if(latest.length) {
        filePath = latest[0].path
      }
    }

    tplUrl = FileParser.getFileDataFromUrl(filePath)
    var fakeContent = (req.query.fakeContent) ? true : false

    obj = editor(templatePath, tplUrl, fakeContent)

    if(!fileUtils.isFile(tplUrl.json.path)) {
      var json = obj.json
      var text = ''
      var tpl = templatePath
      var extension = config.files.templates.extension
      if(fileUtils.isFile(tpl + '.' + extension)) {
        text = fse.readFileSync(tpl + '.' + extension, 'utf8')
      }
      text = Util.removeDataList(text)

      var resSave = save(filePath, req.params[0], json, text, 'draft')
      filePath = resSave.htmlPath
    }

    FileParser.getAssetsFolder()
    FileParser.getAssets() 

    var revisionFilePath = FileParser.changePathEnv(filePath, config.draft.url)
    var dirPath = fileUtils.removeLast(revisionFilePath)
    var allDraft = FileParser.getFiles(dirPath, true, 99, new RegExp("\\." + config.files.templates.extension))

    allDraft = FileParser.getMetas(allDraft, 'draft')
    var breadcrumb = req.params[0].split('/')
    manager.file = {
      revision: fileAttr.getFilesRevision(allDraft, revisionFilePath)
      ,template: breadcrumb
      ,path: (req.query.filePath) ? fileUtils.cleanTplName(req.query.filePath) : ''
    }
    if(manager.file.revision.length > 0){
      var publishPath = fileAttr.delete(manager.file.revision[0].path.replace(new RegExp(`/${config.draft.url}/`), `/${config.publish.url}/`))
      manager.file.isPublished = fileUtils.isFile(publishPath)
    }
  }
  manager.home = {
    files: FileParser.getAllFiles()
  }

  manager.list = FileParser.getProjetFiles()
  manager.editConfig = config.getConfigByWebsite()
  manager.config = JSON.stringify(config)
  
  var _hasBlock = (obj) ? obj.hasBlock : false
  var _hasSingleBlock = (obj) ? obj.hasSingleBlock : false
  var _template = (filePath) ? '/page/' + req.params[0] + `?filePath=${req.query.filePath}` : false
  var _form = (obj) ? obj.form : false
  var _json = (obj) ? obj.json : false
  var _file = (tplUrl) ? tplUrl.draft.file : false
  var _filePath = (req.query.filePath) ? req.query.filePath : false

  // var textMinified = minify(text.replace(/'/g, "\\'"), {collapseWhitespace: true})
  res.render('../views/template-engine', { // render html and send ABE engine to users
    abeUrl: '/abe/',
    test: JSON.stringify(locale),
    text: locale,
    templatePath: req.params[0],
    template: _template,
    hasSingleBlock: _hasSingleBlock,
    hasBlock: _hasBlock,
    form: _form,
    urlToSaveFile: _filePath,
    tplName: _file,
    json: _json,
    config: config,
    manager: manager
  })
})

// Template page route
router.get('/page/*', function(req, res) {
  var templatePath = fileUtils.getTemplatePath(req.params[0])
  var filePath = fileUtils.getFilePath(req.query.filePath)

  if(typeof filePath !== 'undefined' && filePath !== null) {

    var tplUrl = FileParser.getFileDataFromUrl(filePath)
    
    log.write('Index-route-page', 'templatePath', templatePath, 'tplUrl', tplUrl)

    var json = {}

    var resp = fse.readJsonSync(tplUrl.json.path)

    if(typeof resp !== 'undefined' && resp !== null) {
      json = resp
    }
    
    let meta = config.meta.name
    let extension = config.files.templates.extension

    var text = fse.readFileSync(fileUtils.concatPath(config.root, config.templates.url, json[meta].template + '.' + extension), 'utf8')

    Util.getDataList(FileParser.getWebsite(filePath), text, json)
    log.write('data-list', text, json)
    text = Util.removeDataList(text)

    var page = new Page(templatePath, text, json)
    res.set('Content-Type', 'text/html')
    res.send(page.html)
  }else {
    // not 404 page if tag abe image upload into each block
    if(/upload%20image/g.test(req.url)) {
      var b64str = 'data:image/gif;base64,R0lGODlhAQABAAAAACH5BAEKAAEALAAAAAABAAEAAAICTAEAOw=='
      var img = new Buffer(b64str, 'base64');

      res.writeHead(200, {
        'Content-Type': 'image/jpeg',
        'Content-Length': img.length
      });
      res.end(img); 
    }else {
      log.write('Index-route-page', '404 not found', 'filePath', filePath)
      res.status(404).send('Not found');
    }
  }
})

router.post('/save', function(req, res){
  var saved = save(
    fileUtils.getFilePath(req.body.filePath),
    req.body.tplPath,
    req.body.json,
    '',
    req.body.saveAction)
  var result = {
    success: 1,
    json: saved.json
  }
  res.set('Content-Type', 'application/json')
  res.send(JSON.stringify(result))
})

router.get('/save-config', function(req, res){
  config.set(req.query.website, req.query.json)
  res.set('Content-Type', 'application/json')
  res.send(JSON.stringify(req.query.json))
})

router.get('/unpublish', function(req, res){
  var filePath = req.query.filePath
  var dirPath = FileParser.unpublishFile(filePath)

  var result = {
    success: 1,
    file: req.query.filePath
  }
  res.set('Content-Type', 'application/json')
  res.send(JSON.stringify(result))
})

router.get('/delete', function(req, res){
  var filePath = req.query.filePath
  var dirPath = FileParser.deleteFile(filePath)

  var result = {
    success: 1,
    file: req.query.filePath
  }
  res.set('Content-Type', 'application/json')
  res.send(JSON.stringify(result))
})

router.get('/', function(req, res){
  res.redirect('/abe/');
})

router.post('/upload/*', function(req, res){
  var resp = {success: 1}
  var filePath
  var fstream
  var folderWebPath = '/' + config.upload.image
  var folderFilePath = fileUtils.concatPath(config.root, config.publish.url, folderWebPath)
  mkdirp.sync(folderFilePath)

  req.pipe(req.busboy)
  req.busboy.on('file', function (fieldname, file, filename) {
    filePath = fileUtils.concatPath(folderFilePath, filename)
    resp['filePath'] = fileUtils.concatPath(folderWebPath, filename)
    fstream = fs.createWriteStream(filePath)
    file.pipe(fstream)
    fstream.on('close', function () {});
  });
  req.busboy.on('finish', function() {
    res.set('Content-Type', 'application/json')
    res.send(JSON.stringify(resp))
  });
})

var site = new serveSite()

router.get('/site*', function(req, res) {
  if(!site.isStarted) site.start(config)
  res.set('Content-Type', 'application/json')
  res.send(JSON.stringify(site.infos))
})

export default router
